@@ -849,6 +849,14 @@ gck_mock_C_CreateObject (CK_SESSION_HANDLE hSession, CK_ATTRIBUTE_PTR pTemplate,
 	return CKR_OK;
 }
 
+CK_RV
+gck_mock_fail_C_CreateObject (CK_SESSION_HANDLE hSession, CK_ATTRIBUTE_PTR pTemplate,
+                              CK_ULONG ulCount, CK_OBJECT_HANDLE_PTR phObject)
+{
+	/* Always fails */
+	return CKR_FUNCTION_FAILED;
+}
+
 CK_RV
 gck_mock_unsupported_C_CopyObject (CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hObject,
                                    CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
@@ -934,6 +942,13 @@ gck_mock_C_GetAttributeValue (CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hObje
 	return ret;
 }
 
+CK_RV
+gck_mock_fail_C_GetAttributeValue (CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hObject,
+                                   CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount)
+{
+	return CKR_FUNCTION_FAILED;
+}
+
 CK_RV
 gck_mock_C_SetAttributeValue (CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hObject,
                               CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount)
@@ -1017,6 +1032,14 @@ gck_mock_C_FindObjectsInit (CK_SESSION_HANDLE hSession, CK_ATTRIBUTE_PTR pTempla
 	return CKR_OK;
 }
 
+CK_RV
+gck_mock_fail_C_FindObjects (CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE_PTR phObject,
+                             CK_ULONG ulMaxObjectCount, CK_ULONG_PTR pulObjectCount)
+{
+	/* Always fails */
+	return CKR_FUNCTION_FAILED;
+}
+
 CK_RV
 gck_mock_C_FindObjects (CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE_PTR phObject,
                         CK_ULONG ulMaxObjectCount, CK_ULONG_PTR pulObjectCount)
