@@ -26,11 +26,14 @@
 #include "gcr-certificate-chain.h"
 
 #include "gcr-certificate.h"
+#define DEBUG_FLAG GCR_DEBUG_CERTIFICATE_CHAIN
+#include "gcr-debug.h"
 #include "gcr-pkcs11-certificate.h"
 #include "gcr-simple-certificate.h"
-
 #include "gcr-trust.h"
 
+#include "egg/egg-error.h"
+
 /**
  * SECTION:gcr-certificate-chain
  * @title: GcrCertificateChain
@@ -67,6 +70,26 @@
  * merely the first step towards verifying trust in a certificate.
  */
 
+
+/**
+ * GCR_TYPE_CERTIFICATE_CHAIN_STATUS:
+ *
+ * The flags #GType for #GcrCertificateChainFlags.
+ */
+
+/**
+ * GcrCertificateChain:
+ *
+ * A chain of certificates.
+ */
+
+/**
+ * GcrCertificateChainClass:
+ * @parent_class: The parent class
+ *
+ * The class for #GcrCertificateChain.
+ */
+
 enum {
 	PROP_0,
 	PROP_STATUS,
@@ -162,6 +185,8 @@ prep_chain_private_thread_safe (GcrCertificateChainPrivate *orig, const gchar *p
 			g_return_val_if_fail (der, NULL);
 			safe = gcr_simple_certificate_new (der, n_der);
 
+			_gcr_debug ("copying certificate so it's thread safe");
+
 			/* Always set the original certificate onto the safe one */
 			g_object_set_qdata_full (G_OBJECT (safe), Q_ORIGINAL_CERT,
 			                         g_object_ref (certificate), g_object_unref);
@@ -203,6 +228,7 @@ perform_build_chain (GcrCertificateChainPrivate *pv, GCancellable *cancellable,
 	gboolean lookups;
 	gboolean ret;
 	guint length;
+	gchar *subject;
 
 	g_assert (pv);
 	g_assert (pv->certificates);
@@ -211,15 +237,25 @@ perform_build_chain (GcrCertificateChainPrivate *pv, GCancellable *cancellable,
 	lookups = !((pv->flags & GCR_CERTIFICATE_CHAIN_FLAG_NO_LOOKUPS) == GCR_CERTIFICATE_CHAIN_FLAG_NO_LOOKUPS);
 
 	/* This chain is built */
-	if (!pv->certificates->len)
+	if (!pv->certificates->len) {
+		_gcr_debug ("empty certificate chain");
 		return TRUE;
+	}
 
 	/* First check for pinned certificates */
 	certificate = g_ptr_array_index (pv->certificates, 0);
+	if (_gcr_debugging) {
+		subject = gcr_certificate_get_subject_dn (certificate);
+		_gcr_debug ("first certificate: %s", subject);
+		g_free (subject);
+	}
+
 	if (lookups && pv->peer) {
 		ret = gcr_trust_is_certificate_pinned (certificate, pv->purpose,
 		                                       pv->peer, cancellable, &error);
 		if (!ret && error) {
+			_gcr_debug ("failed to lookup pinned certificate: %s",
+			            egg_error_message (error));
 			g_propagate_error (rerror, error);
 			return FALSE;
 		}
@@ -229,6 +265,9 @@ perform_build_chain (GcrCertificateChainPrivate *pv, GCancellable *cancellable,
 		 * is irrelevant, so truncate chain and consider built.
 		 */
 		if (ret) {
+			_gcr_debug ("found pinned certificate for peer '%s', truncating chain",
+			            pv->peer);
+
 			g_ptr_array_set_size (pv->certificates, 1);
 			pv->status = GCR_CERTIFICATE_CHAIN_PINNED;
 			return TRUE;
@@ -242,6 +281,7 @@ perform_build_chain (GcrCertificateChainPrivate *pv, GCancellable *cancellable,
 
 		/* Stop the chain if previous was self-signed */
 		if (gcr_certificate_is_issuer (certificate, certificate)) {
+			_gcr_debug ("found self-signed certificate");
 			pv->status = GCR_CERTIFICATE_CHAIN_SELFSIGNED;
 			break;
 		}
@@ -249,25 +289,42 @@ perform_build_chain (GcrCertificateChainPrivate *pv, GCancellable *cancellable,
 		/* Try the next certificate in the chain */
 		if (length < pv->certificates->len) {
 			certificate = g_ptr_array_index (pv->certificates, length);
+			if (_gcr_debugging) {
+				subject = gcr_certificate_get_subject_dn (certificate);
+				_gcr_debug ("next certificate: %s", subject);
+				g_free (subject);
+			}
 
 		/* No more in chain, try to lookup */
 		} else if (lookups) {
 			certificate = gcr_pkcs11_certificate_lookup_issuer (certificate,
 			                                                    cancellable, &error);
 			if (error != NULL) {
+				_gcr_debug ("failed to lookup issuer: %s", error->message);
 				g_propagate_error (rerror, error);
 				return FALSE;
+
 			} else if (certificate) {
 				g_ptr_array_add (pv->certificates, certificate);
+				if (_gcr_debugging) {
+					subject = gcr_certificate_get_subject_dn (certificate);
+					_gcr_debug ("found issuer certificate: %s", subject);
+					g_free (subject);
+				}
+
+			} else {
+				_gcr_debug ("no issuer found");
 			}
 
 		/* No more in chain, and can't lookup */
 		} else {
+			_gcr_debug ("no more certificates available, and no lookups");
 			certificate = NULL;
 		}
 
 		/* Stop the chain if nothing found */
 		if (certificate == NULL) {
+			_gcr_debug ("chain is incomplete");
 			pv->status = GCR_CERTIFICATE_CHAIN_INCOMPLETE;
 			break;
 		}
@@ -280,11 +337,14 @@ perform_build_chain (GcrCertificateChainPrivate *pv, GCancellable *cancellable,
 			                                         cancellable, &error);
 
 			if (!ret && error) {
+				_gcr_debug ("failed to lookup anchored certificate: %s",
+				            egg_error_message (error));
 				g_propagate_error (rerror, error);
 				return FALSE;
 
 			/* Stop the chain at the first anchor */
 			} else if (ret) {
+				_gcr_debug ("found anchored certificate");
 				pv->status = GCR_CERTIFICATE_CHAIN_ANCHORED;
 				break;
 			}
@@ -309,6 +369,8 @@ thread_build_chain (GSimpleAsyncResult *result, GObject *object,
 	pv = g_object_get_qdata (G_OBJECT (result), Q_OPERATION_DATA);
 	g_assert (pv);
 
+	_gcr_debug ("building asynchronously in another thread");
+
 	if (!perform_build_chain (pv, cancellable, &error)) {
 		g_simple_async_result_set_from_error (result, error);
 		g_clear_error (&error);
@@ -428,6 +490,12 @@ gcr_certificate_chain_class_init (GcrCertificateChainClass *klass)
  * built.
  */
 
+/**
+ * GCR_TYPE_CERTIFICATE_CHAIN_STATUS:
+ *
+ * The enum #GType for #GcrCertificateChainStatus.
+ */
+
 GType
 gcr_certificate_chain_status_get_type (void)
 {
@@ -461,6 +529,12 @@ gcr_certificate_chain_status_get_type (void)
  * Flags to be used with the gcr_certificate_chain_build() operation.
  */
 
+/**
+ * GCR_TYPE_CERTIFICATE_CHAIN_FLAGS:
+ *
+ * The flags #GType for #GcrCertificateChainFlags.
+ */
+
 GType
 gcr_certificate_chain_flags_get_type (void)
 {
