@@ -24,7 +24,8 @@
 #include "gcr-internal.h"
 #include "gcr-certificate.h"
 
-#include "egg/egg-asn1.h"
+#include "egg/egg-asn1x.h"
+#include "egg/egg-asn1-defs.h"
 #include "egg/egg-dn.h"
 #include "egg/egg-hex.h"
 
@@ -33,13 +34,30 @@
 /**
  * SECTION:gcr-certificate
  * @title: GcrCertificate
- * @short_description: Represents a certificate.
+ * @short_description: Represents an X.509 certificate
  * 
- * This is an interface that represents an X509 certificate. Objects can 
+ * This is an interface that represents an X.509 certificate. Objects can
  * implement this interface to make a certificate usable with the GCR
  * library. 
- * 
- * You can use #GcrSimpleCertificate to simply load a certificate.
+ *
+ * Various methods are available to parse out relevant bits of the certificate.
+ * However no verification of the validity of a certificate is done here. Use
+ * your favorite crypto library to do this.
+ *
+ * You can use #GcrSimpleCertificate to simply load a certificate for which
+ * you already have the raw certificate data.
+ */
+
+/**
+ * GcrCertificate:
+ *
+ * An object which holds a certificate.
+ */
+
+/**
+ * GcrCertificateIface:
+ *
+ * The interface that implementors of #GcrCertificate must implement.
  */
 
 /* 
@@ -54,12 +72,17 @@
  */
 
 typedef struct _GcrCertificateInfo {
-	const guchar *der;
+	gconstpointer der;
 	gsize n_der;
-	ASN1_TYPE asn1;
+	GNode *asn1;
 	guint key_size;
 } GcrCertificateInfo;
 
+/* Forward declarations */
+
+static gconstpointer _gcr_certificate_get_subject_const (GcrCertificate *self, gsize *n_data);
+static gconstpointer _gcr_certificate_get_issuer_const (GcrCertificate *self, gsize *n_data);
+
 /* -----------------------------------------------------------------------------
  * INTERNAL 
  */
@@ -74,7 +97,7 @@ certificate_info_free (gpointer data)
 	GcrCertificateInfo *info = data;
 	if (info) {
 		g_assert (info->asn1);
-		asn1_delete_structure (&info->asn1);
+		egg_asn1x_destroy (info->asn1);
 		g_free (info);
 	}
 }
@@ -83,12 +106,12 @@ static GcrCertificateInfo*
 certificate_info_load (GcrCertificate *cert)
 {
 	GcrCertificateInfo *info;
-	ASN1_TYPE asn1;
-	const guchar *der;
+	GNode *asn1;
+	gconstpointer der;
 	gsize n_der;
-	
+
 	g_assert (GCR_IS_CERTIFICATE (cert));
-	
+
 	der = gcr_certificate_get_der_data (cert, &n_der);
 	g_return_val_if_fail (der, NULL);
 
@@ -99,9 +122,9 @@ certificate_info_load (GcrCertificate *cert)
 	}
 	
 	/* Cache is invalid or non existent */
-	asn1 = egg_asn1_decode ("PKIX1.Certificate", der, n_der);
+	asn1 = egg_asn1x_create_and_decode (pkix_asn1_tab, "Certificate", der, n_der);
 	if (asn1 == NULL) {
-		g_warning ("a derived class provided an invalid or unparseable X509 DER certificate data.");
+		g_warning ("a derived class provided an invalid or unparseable X.509 DER certificate data.");
 		return NULL;
 	}
 	
@@ -109,43 +132,43 @@ certificate_info_load (GcrCertificate *cert)
 	info->der = der;
 	info->n_der = n_der;
 	info->asn1 = asn1;
-	
+
 	g_object_set_qdata_full (G_OBJECT (cert), CERTIFICATE_INFO, info, certificate_info_free);
 	return info;
 }
 
 static guint
-calculate_rsa_key_size (const guchar *data, gsize n_data)
+calculate_rsa_key_size (gconstpointer data, gsize n_data)
 {
-	ASN1_TYPE asn;
+	GNode *asn;
 	gsize n_content;
-	
-	asn = egg_asn1_decode ("PK.RSAPublicKey", data, n_data);
+
+	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "RSAPublicKey", data, n_data);
 	g_return_val_if_fail (asn, 0);
-    
-	if (!egg_asn1_read_content (asn, data, n_data, "modulus", &n_content))
+
+	if (!egg_asn1x_get_raw_value (egg_asn1x_node (asn, "modulus", NULL), &n_content))
 		g_return_val_if_reached (0);
-	
-	asn1_delete_structure (&asn);
-	
+
+	egg_asn1x_destroy (asn);
+
 	/* Removes the complement */
 	return (n_content / 2) * 2 * 8;
 }
 
 static guint
-calculate_dsa_params_size (const guchar *data, gsize n_data)
+calculate_dsa_params_size (gconstpointer data, gsize n_data)
 {
-	ASN1_TYPE asn;
+	GNode *asn;
 	gsize n_content;
-	
-	asn = egg_asn1_decode ("PK.DSAParameters", data, n_data);
+
+	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "DSAParameters", data, n_data);
 	g_return_val_if_fail (asn, 0);
 
-	if (!egg_asn1_read_content (asn, data, n_data, "p", &n_content))
+	if (!egg_asn1x_get_raw_value (egg_asn1x_node (asn, "p", NULL), &n_content))
 		g_return_val_if_reached (0);
-		
-	asn1_delete_structure (&asn);
-	
+
+	egg_asn1x_destroy (asn);
+
 	/* Removes the complement */
 	return (n_content / 2) * 2 * 8;
 }
@@ -153,43 +176,43 @@ calculate_dsa_params_size (const guchar *data, gsize n_data)
 static guint
 calculate_key_size (GcrCertificateInfo *info)
 {
-	ASN1_TYPE asn;
-	const guchar *data, *params;
-	gsize n_data, n_params, n_key;
-	guint key_size = 0;
-	guchar *key;
+	GNode *asn;
+	gconstpointer data, params;
+	gsize n_data, n_params;
+	guint key_size = 0, n_bits;
+	guchar *key = NULL;
 	GQuark oid;
-	
-	data = egg_asn1_read_element (info->asn1, info->der, info->n_der, "tbsCertificate.subjectPublicKeyInfo", &n_data);
+
+	data = egg_asn1x_get_raw_element (egg_asn1x_node (info->asn1, "tbsCertificate", "subjectPublicKeyInfo", NULL), &n_data);
 	g_return_val_if_fail (data != NULL, 0);
-	
-	asn = egg_asn1_decode ("PKIX1.SubjectPublicKeyInfo", data, n_data);
+
+	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "SubjectPublicKeyInfo", data, n_data);
 	g_return_val_if_fail (asn, 0);
-	
+
 	/* Figure out the algorithm */
-	oid = egg_asn1_read_oid (asn, "algorithm.algorithm");
+	oid = egg_asn1x_get_oid_as_quark (egg_asn1x_node (asn, "algorithm", "algorithm", NULL));
 	g_return_val_if_fail (oid, 0);
-		
+
 	/* RSA keys are stored in the main subjectPublicKey field */
 	if (oid == OID_RSA_KEY) {
-		
+
 		/* A bit string so we cannot process in place */
-		key = egg_asn1_read_value (asn, "subjectPublicKey", &n_key, NULL);
+		key = egg_asn1x_get_bits_as_raw (egg_asn1x_node (asn, "subjectPublicKey", NULL), NULL, &n_bits);
 		g_return_val_if_fail (key, 0);
-		key_size = calculate_rsa_key_size (key, n_key / 8);
+		key_size = calculate_rsa_key_size (key, n_bits / 8);
+		g_free (key);
 
 	/* The DSA key size is discovered by the prime in params */
 	} else if (oid == OID_DSA_KEY) {
-		params = egg_asn1_read_element (asn, data, n_data, "algorithm.parameters", &n_params);
+		params = egg_asn1x_get_raw_element (egg_asn1x_node (asn, "algorithm", "parameters", NULL), &n_params);
 		key_size = calculate_dsa_params_size (params, n_params);
-		
+
 	} else {
 		g_message ("unsupported key algorithm in certificate: %s", g_quark_to_string (oid));
 	}
-	
-	asn1_delete_structure (&asn);
-	g_free (key);
-	
+
+	egg_asn1x_destroy (asn);
+
 	return key_size;
 }
 
@@ -197,9 +220,9 @@ static GChecksum*
 digest_certificate (GcrCertificate *self, GChecksumType type)
 {
 	GChecksum *digest;
-	const guchar *der;
+	gconstpointer der;
 	gsize n_der;
-	
+
 	g_assert (GCR_IS_CERTIFICATE (self));
 
 	der = gcr_certificate_get_der_data (self, &n_der);
@@ -265,11 +288,11 @@ gcr_certificate_get_type (void)
  * @self: a #GcrCertificate
  * @n_data: a pointer to a location to store the size of the resulting DER data.
  * 
- * Gets the raw DER data for an X509 certificate.
+ * Gets the raw DER data for an X.509 certificate.
  * 
- * Returns: raw DER data of the X509 certificate.
+ * Returns: raw DER data of the X.509 certificate.
  **/
-const guchar*
+gconstpointer
 gcr_certificate_get_der_data (GcrCertificate *self, gsize *n_data)
 {
 	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), NULL);
@@ -319,8 +342,73 @@ gcr_certificate_get_issuer_part (GcrCertificate *self, const char *part)
 	
 	info = certificate_info_load (self);
 	g_return_val_if_fail (info, NULL);
-	
-	return egg_dn_read_part (info->asn1, "tbsCertificate.issuer.rdnSequence", part);
+
+	return egg_dn_read_part (egg_asn1x_node (info->asn1, "tbsCertificate", "issuer", "rdnSequence", NULL), part);
+}
+
+static gconstpointer
+_gcr_certificate_get_issuer_const (GcrCertificate *self, gsize *n_data)
+{
+	GcrCertificateInfo *info;
+
+	info = certificate_info_load (self);
+	g_return_val_if_fail (info, NULL);
+
+	return egg_asn1x_get_raw_element (egg_asn1x_node (info->asn1, "tbsCertificate", "issuer", NULL), n_data);
+}
+
+/**
+ * gcr_certificate_get_issuer_raw:
+ * @self: a #GcrCertificate
+ * @n_data: The length of the returned data.
+ *
+ * Get the raw DER data for the issuer DN of the certificate.
+ *
+ * The data should be freed by using g_free() when no longer required.
+ *
+ * Returns: allocated memory containing the raw issuer.
+ */
+gpointer
+gcr_certificate_get_issuer_raw (GcrCertificate *self, gsize *n_data)
+{
+	gconstpointer data;
+
+	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), NULL);
+	g_return_val_if_fail (n_data, NULL);
+
+	data = _gcr_certificate_get_issuer_const (self, n_data);
+	return g_memdup (data, data ? *n_data : 0);
+}
+
+/**
+ * gcr_certificate_is_issuer:
+ * @self: a #GcrCertificate
+ * @issuer: a possible issuer #GcrCertificate
+ *
+ * Check if @issuer could be the issuer of this certificate. This is done by
+ * comparing the relevant subject and issuer fields. No signature check is
+ * done. Proper verification of certificates must be done via a crypto
+ * library.
+ *
+ * Returns: whether @issuer could be the issuer of the certificate.
+ */
+gboolean
+gcr_certificate_is_issuer (GcrCertificate *self, GcrCertificate *issuer)
+{
+	gconstpointer subject_dn, issuer_dn;
+	gsize n_subject_dn, n_issuer_dn;
+
+	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), FALSE);
+	g_return_val_if_fail (GCR_IS_CERTIFICATE (issuer), FALSE);
+
+	subject_dn = _gcr_certificate_get_subject_const (issuer, &n_subject_dn);
+	g_return_val_if_fail (subject_dn, FALSE);
+
+	issuer_dn = _gcr_certificate_get_issuer_const (self, &n_issuer_dn);
+	g_return_val_if_fail (issuer_dn, FALSE);
+
+	return (n_issuer_dn == n_subject_dn &&
+	        memcmp (issuer_dn, subject_dn, n_issuer_dn) == 0);
 }
 
 /**
@@ -344,8 +432,8 @@ gcr_certificate_get_issuer_dn (GcrCertificate *self)
 	
 	info = certificate_info_load (self);
 	g_return_val_if_fail (info, NULL);
-	
-	return egg_dn_read (info->asn1, "tbsCertificate.issuer.rdnSequence");
+
+	return egg_dn_read (egg_asn1x_node (info->asn1, "tbsCertificate", "issuer", "rdnSequence", NULL));
 }
 
 /**
@@ -390,8 +478,8 @@ gcr_certificate_get_subject_part (GcrCertificate *self, const char *part)
 	
 	info = certificate_info_load (self);
 	g_return_val_if_fail (info, NULL);
-	
-	return egg_dn_read_part (info->asn1, "tbsCertificate.subject.rdnSequence", part);
+
+	return egg_dn_read_part (egg_asn1x_node (info->asn1, "tbsCertificate", "subject", "rdnSequence", NULL), part);
 }
 
 /**
@@ -415,8 +503,46 @@ gcr_certificate_get_subject_dn (GcrCertificate *self)
 	
 	info = certificate_info_load (self);
 	g_return_val_if_fail (info, NULL);
-	
-	return egg_dn_read (info->asn1, "tbsCertificate.issuer.rdnSequence");
+
+	return egg_dn_read (egg_asn1x_node (info->asn1, "tbsCertificate", "subject", "rdnSequence", NULL));
+}
+
+static gconstpointer
+_gcr_certificate_get_subject_const (GcrCertificate *self, gsize *n_data)
+{
+	GcrCertificateInfo *info;
+
+	info = certificate_info_load (self);
+	g_return_val_if_fail (info, NULL);
+
+	return egg_asn1x_get_raw_element (egg_asn1x_node (info->asn1, "tbsCertificate", "subject", NULL), n_data);
+}
+
+/**
+ * gcr_certificate_get_subject_raw:
+ * @self: a #GcrCertificate
+ * @n_data: The length of the returned data.
+ *
+ * Get the raw DER data for the subject DN of the certificate.
+ *
+ * The data should be freed by using g_free() when no longer required.
+ *
+ * Returns: allocated memory containing the raw subject.
+ */
+gpointer
+gcr_certificate_get_subject_raw (GcrCertificate *self, gsize *n_data)
+{
+	GcrCertificateInfo *info;
+	gconstpointer data;
+
+	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), NULL);
+	g_return_val_if_fail (n_data, NULL);
+
+	info = certificate_info_load (self);
+	g_return_val_if_fail (info, NULL);
+
+	data = _gcr_certificate_get_subject_const (self, n_data);
+	return g_memdup (data, data ? *n_data : 0);
 }
 
 /**
@@ -442,7 +568,7 @@ gcr_certificate_get_issued_date (GcrCertificate *self)
 	g_return_val_if_fail (info, NULL);
 	
 	date = g_date_new ();
-	if (!egg_asn1_read_date (info->asn1, "tbsCertificate.validity.notBefore", date)) {
+	if (!egg_asn1x_get_time_as_date (egg_asn1x_node (info->asn1, "tbsCertificate", "validity", "notBefore", NULL), date)) {
 		g_date_free (date);
 		return NULL;
 	}
@@ -473,7 +599,7 @@ gcr_certificate_get_expiry_date (GcrCertificate *self)
 	g_return_val_if_fail (info, NULL);
 	
 	date = g_date_new ();
-	if (!egg_asn1_read_date (info->asn1, "tbsCertificate.validity.notAfter", date)) {
+	if (!egg_asn1x_get_time_as_date (egg_asn1x_node (info->asn1, "tbsCertificate", "validity", "notAfter", NULL), date)) {
 		g_date_free (date);
 		return NULL;
 	}
@@ -600,14 +726,14 @@ guchar*
 gcr_certificate_get_serial_number (GcrCertificate *self, gsize *n_length)
 {
 	GcrCertificateInfo *info;
-	
+
 	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), NULL);
 	g_return_val_if_fail (n_length, NULL);
 	
 	info = certificate_info_load (self);
 	g_return_val_if_fail (info, NULL);
-	
-	return egg_asn1_read_value (info->asn1, "tbsCertificate.serialNumber", n_length, g_realloc); 
+
+	return egg_asn1x_get_integer_as_raw (egg_asn1x_node (info->asn1, "tbsCertificate", "serialNumber", NULL), NULL, n_length);
 }
 
 /**
