@@ -22,14 +22,14 @@
 
 #include "config.h"
 
-#include "test-suite.h"
-
 #include "egg/egg-asn1x.h"
 #include "egg/egg-asn1-defs.h"
 
-#include "gcr.h"
+#include "gcr/gcr.h"
 #include "gcr/gcr-internal.h"
 
+#include "egg/egg-testing.h"
+
 #include "gck/gck-mock.h"
 #include "gck/gck-test.h"
 
@@ -37,13 +37,18 @@
 
 #include <glib.h>
 
-static gpointer cert_data = NULL;
-static gsize n_cert_data = 0;
-static gpointer cert2_data = NULL;
-static gsize n_cert2_data = 0;
-static CK_FUNCTION_LIST funcs;
+#include <errno.h>
 
-TESTING_SETUP (pkcs11_certificate)
+typedef struct {
+	gpointer cert_data;
+	gsize n_cert_data;
+	gpointer cert2_data;
+	gsize n_cert2_data;
+	CK_FUNCTION_LIST funcs;
+} Test;
+
+static void
+setup (Test *test, gconstpointer unused)
 {
 	GList *modules = NULL;
 	GckAttributes *attrs;
@@ -54,34 +59,39 @@ TESTING_SETUP (pkcs11_certificate)
 	GNode *asn, *node;
 	CK_RV rv;
 
-	cert_data = testing_data_read ("der-certificate.crt", &n_cert_data);
-	g_assert (cert_data);
+	if (!g_file_get_contents (SRCDIR "/files/der-certificate.crt", (gchar**)&test->cert_data,
+	                          &test->n_cert_data, NULL))
+		g_assert_not_reached ();
+	g_assert (test->cert_data);
 
-	cert2_data = testing_data_read ("der-certificate-dsa.cer", &n_cert2_data);
-	g_assert (cert2_data);
+	if (!g_file_get_contents (SRCDIR "/files/der-certificate-dsa.cer", (gchar**)&test->cert2_data,
+	                          &test->n_cert2_data, NULL))
+		g_assert_not_reached ();
+	g_assert (test->cert2_data);
 
 	rv = gck_mock_C_GetFunctionList (&f);
 	gck_assert_cmprv (rv, ==, CKR_OK);
-	memcpy (&funcs, f, sizeof (funcs));
+	memcpy (&test->funcs, f, sizeof (test->funcs));
 
 	/* Open a session */
-	rv = (funcs.C_Initialize) (NULL);
+	rv = (test->funcs.C_Initialize) (NULL);
 	gck_assert_cmprv (rv, ==, CKR_OK);
 
 	g_assert (!modules);
-	module = gck_module_new (&funcs, 0);
+	module = gck_module_new (&test->funcs, 0);
 	modules = g_list_prepend (modules, module);
 	gcr_pkcs11_set_modules (modules);
 	gck_list_unref_free (modules);
 
-	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "Certificate", cert_data, n_cert_data);
+	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "Certificate",
+	                                   test->cert_data, test->n_cert_data);
 	g_assert (asn);
 	node = egg_asn1x_node (asn, "tbsCertificate", "subject", NULL);
 	subject = egg_asn1x_get_raw_element (node, &n_subject);
 
 	/* Add a certificate to the module */
 	attrs = gck_attributes_new ();
-	gck_attributes_add_data (attrs, CKA_VALUE, cert_data, n_cert_data);
+	gck_attributes_add_data (attrs, CKA_VALUE, test->cert_data, test->n_cert_data);
 	gck_attributes_add_ulong (attrs, CKA_CLASS, CKO_CERTIFICATE);
 	gck_attributes_add_ulong (attrs, CKA_CERTIFICATE_TYPE, CKC_X_509);
 	gck_attributes_add_data (attrs, CKA_SUBJECT, subject, n_subject);
@@ -90,23 +100,20 @@ TESTING_SETUP (pkcs11_certificate)
 	egg_asn1x_destroy (asn);
 }
 
-TESTING_TEARDOWN (pkcs11_certificate)
+static void
+teardown (Test *test, gconstpointer unused)
 {
 	CK_RV rv;
 
-	g_free (cert_data);
-	cert_data = NULL;
-	n_cert_data = 0;
+	g_free (test->cert_data);
+	g_free (test->cert2_data);
 
-	g_free (cert2_data);
-	cert2_data = NULL;
-	n_cert2_data = 0;
-
-	rv = (funcs.C_Finalize) (NULL);
+	rv = (test->funcs.C_Finalize) (NULL);
 	gck_assert_cmprv (rv, ==, CKR_OK);
 }
 
-TESTING_TEST (pkcs11_lookup_certificate_issuer)
+static void
+test_lookup_certificate_issuer (Test *test, gconstpointer unused)
 {
 	GcrCertificate *cert, *issuer;
 	GError *error = NULL;
@@ -115,7 +122,7 @@ TESTING_TEST (pkcs11_lookup_certificate_issuer)
 	gconstpointer der;
 	gsize n_der;
 
-	cert = gcr_simple_certificate_new_static (cert_data, n_cert_data);
+	cert = gcr_simple_certificate_new_static (test->cert_data, test->n_cert_data);
 	g_assert (cert);
 
 	/* Should be self-signed, so should find itself (added in setup) */
@@ -125,16 +132,16 @@ TESTING_TEST (pkcs11_lookup_certificate_issuer)
 
 	/* Should be the same certificate */
 	der = gcr_certificate_get_der_data (issuer, &n_der);
-	g_assert_cmpsize (n_der, ==, n_cert_data);
-	g_assert (memcmp (der, cert_data, n_cert_data) == 0);
+	egg_assert_cmpsize (n_der, ==, test->n_cert_data);
+	g_assert (memcmp (der, test->cert_data, test->n_cert_data) == 0);
 
 	/* Should return the same certificate here too */
 	attrs = gcr_pkcs11_certificate_get_attributes (GCR_PKCS11_CERTIFICATE (issuer));
 	g_assert (attrs);
 	attr = gck_attributes_find (attrs, CKA_VALUE);
 	g_assert (attr);
-	g_assert_cmpsize (attr->length, ==, n_cert_data);
-	g_assert (memcmp (attr->value, cert_data, n_cert_data) == 0);
+	egg_assert_cmpsize (attr->length, ==, test->n_cert_data);
+	g_assert (memcmp (attr->value, test->cert_data, test->n_cert_data) == 0);
 
 	/* Should return the same certificate here too */
 	attrs = NULL;
@@ -142,20 +149,21 @@ TESTING_TEST (pkcs11_lookup_certificate_issuer)
 	g_assert (attrs);
 	attr = gck_attributes_find (attrs, CKA_VALUE);
 	g_assert (attr);
-	g_assert_cmpsize (attr->length, ==, n_cert_data);
-	g_assert (memcmp (attr->value, cert_data, n_cert_data) == 0);
+	egg_assert_cmpsize (attr->length, ==, test->n_cert_data);
+	g_assert (memcmp (attr->value, test->cert_data, test->n_cert_data) == 0);
 	gck_attributes_unref (attrs);
 
 	g_object_unref (cert);
 	g_object_unref (issuer);
 }
 
-TESTING_TEST (pkcs11_lookup_certificate_issuer_not_found)
+static void
+test_lookup_certificate_issuer_not_found (Test *test, gconstpointer unused)
 {
 	GcrCertificate *cert, *issuer;
 	GError *error = NULL;
 
-	cert = gcr_simple_certificate_new_static (cert2_data, n_cert2_data);
+	cert = gcr_simple_certificate_new_static (test->cert2_data, test->n_cert2_data);
 	g_assert (cert);
 
 	/* Issuer shouldn't be found */
@@ -171,10 +179,11 @@ fetch_async_result (GObject *source, GAsyncResult *result, gpointer user_data)
 {
 	*((GAsyncResult**)user_data) = result;
 	g_object_ref (result);
-	testing_wait_stop ();
+	egg_test_wait_stop ();
 }
 
-TESTING_TEST (pkcs11_lookup_certificate_issuer_async)
+static void
+test_lookup_certificate_issuer_async (Test *test, gconstpointer unused)
 {
 	GAsyncResult *result = NULL;
 	GcrCertificate *cert, *issuer;
@@ -182,12 +191,12 @@ TESTING_TEST (pkcs11_lookup_certificate_issuer_async)
 	gconstpointer der;
 	gsize n_der;
 
-	cert = gcr_simple_certificate_new_static (cert_data, n_cert_data);
+	cert = gcr_simple_certificate_new_static (test->cert_data, test->n_cert_data);
 	g_assert (cert);
 
 	/* Should be self-signed, so should find itself (added in setup) */
 	gcr_pkcs11_certificate_lookup_issuer_async (cert, NULL, fetch_async_result, &result);
-	testing_wait_until (500);
+	egg_test_wait_until (500);
 	g_assert (result);
 	issuer = gcr_pkcs11_certificate_lookup_issuer_finish (result, &error);
 	g_assert (GCR_IS_PKCS11_CERTIFICATE (issuer));
@@ -197,23 +206,24 @@ TESTING_TEST (pkcs11_lookup_certificate_issuer_async)
 
 	/* Should be the same certificate */
 	der = gcr_certificate_get_der_data (issuer, &n_der);
-	g_assert_cmpsize (n_der, ==, n_cert_data);
-	g_assert (memcmp (der, cert_data, n_cert_data) == 0);
+	egg_assert_cmpsize (n_der, ==, test->n_cert_data);
+	g_assert (memcmp (der, test->cert_data, test->n_cert_data) == 0);
 
 	g_object_unref (cert);
 	g_object_unref (issuer);
 }
 
-TESTING_TEST (pkcs11_lookup_certificate_issuer_failure)
+static void
+test_lookup_certificate_issuer_failure (Test *test, gconstpointer unused)
 {
 	GcrCertificate *cert, *issuer;
 	GError *error = NULL;
 
-	cert = gcr_simple_certificate_new_static (cert_data, n_cert_data);
+	cert = gcr_simple_certificate_new_static (test->cert_data, test->n_cert_data);
 	g_assert (cert);
 
 	/* Make the lookup fail */
-	funcs.C_GetAttributeValue = gck_mock_fail_C_GetAttributeValue;
+	test->funcs.C_GetAttributeValue = gck_mock_fail_C_GetAttributeValue;
 
 	issuer = gcr_pkcs11_certificate_lookup_issuer (cert, NULL, &error);
 	g_assert (issuer == NULL);
@@ -224,21 +234,22 @@ TESTING_TEST (pkcs11_lookup_certificate_issuer_failure)
 	g_object_unref (cert);
 }
 
-TESTING_TEST (pkcs11_lookup_certificate_issuer_fail_async)
+static void
+test_lookup_certificate_issuer_fail_async (Test *test, gconstpointer unused)
 {
 	GAsyncResult *result = NULL;
 	GcrCertificate *cert, *issuer;
 	GError *error = NULL;
 
-	cert = gcr_simple_certificate_new_static (cert_data, n_cert_data);
+	cert = gcr_simple_certificate_new_static (test->cert_data, test->n_cert_data);
 	g_assert (cert);
 
 	/* Make the lookup fail */
-	funcs.C_GetAttributeValue = gck_mock_fail_C_GetAttributeValue;
+	test->funcs.C_GetAttributeValue = gck_mock_fail_C_GetAttributeValue;
 
 	/* Should be self-signed, so should find itself (added in setup) */
 	gcr_pkcs11_certificate_lookup_issuer_async (cert, NULL, fetch_async_result, &result);
-	testing_wait_until (500);
+	egg_test_wait_until (500);
 	g_assert (result);
 	issuer = gcr_pkcs11_certificate_lookup_issuer_finish (result, &error);
 	g_assert (issuer == NULL);
@@ -250,3 +261,19 @@ TESTING_TEST (pkcs11_lookup_certificate_issuer_fail_async)
 
 	g_object_unref (cert);
 }
+
+int
+main (int argc, char **argv)
+{
+	g_type_init ();
+	g_test_init (&argc, &argv, NULL);
+	g_set_prgname ("test-pkcs11-certificate");
+
+	g_test_add ("/gcr/pkcs11-certificate/lookup_certificate_issuer", Test, NULL, setup, test_lookup_certificate_issuer, teardown);
+	g_test_add ("/gcr/pkcs11-certificate/lookup_certificate_issuer_not_found", Test, NULL, setup, test_lookup_certificate_issuer_not_found, teardown);
+	g_test_add ("/gcr/pkcs11-certificate/lookup_certificate_issuer_async", Test, NULL, setup, test_lookup_certificate_issuer_async, teardown);
+	g_test_add ("/gcr/pkcs11-certificate/lookup_certificate_issuer_failure", Test, NULL, setup, test_lookup_certificate_issuer_failure, teardown);
+	g_test_add ("/gcr/pkcs11-certificate/lookup_certificate_issuer_fail_async", Test, NULL, setup, test_lookup_certificate_issuer_fail_async, teardown);
+
+	return egg_tests_run_in_thread_with_loop ();
+}
